package org.elsysbg.objectDetectionTools;

import java.awt.Color;

public class LABColor {
	private double l;
	private double a;
	private double b;
	
/**
 * 	Construct a new color in the LAB color space
 * @param l dimension l for lightness
 * @param a color-opponent dimension
 * @param b color-opponent dimension
 */
	public LABColor(double l, double a, double b) {
		this.l = l;
		this.a = a;
		this.b = b;
	}
	
	public LABColor(int red, int green, int blue){
		this(new Color(red,green,blue));
	}
	public LABColor(Color color){
		this(new XYZColor(color));
	}
	
	/**
	 * l = 116*f(Y/Yn) - 16
	 * a = 500* [f(X/Xn) - (Y/Yn)]
	 * b = 200* [f(Y/Yn) - f(Z/Zn)]
	 * @param xyzColor
	 */
	public LABColor(XYZColor xyzColor){
		double labX = xyzToLAB(xyzColor.getX()/95.047);
		double labY = xyzToLAB(xyzColor.getY()/100.);
		double labZ = xyzToLAB(xyzColor.getZ()/108.883);
		
		this.l = 116*labY - 16;
		this.a = 500*(labX - labY);
		this.b = 200*(labY - labZ);
	}

	/** 
	 * this method converts color from xyz color space to LAB 
	 * color space
	 * @param value
	 * @return
	 */
	//http://www.brucelindbloom.com/index.html?Eqn_XYZ_to_Lab.html
	double xyzToLAB(double value) {
		if (value > 216. / 24389){ // epsilon
			value = Math.pow(value, 1. / 3);
		}else {
			value = ((24389. / 27) * value + 16.) / 116; // k
		}
		return value;
	}
	
	/**
	 * this method returns the difference between current color
	 * and given color according to the convention from 1976 
	 * @param color
	 * @return
	 */
	public double getDifference1976(LABColor color) {
		double dl = this.l - color.l;
		double da = this.a - color.a;
		double db = this.b - color.b;

		return Math.sqrt(dl * dl + da * da + db * db);
	}

	public double getL() {
		return l;
	}

	public double getA() {
		return a;
	}

	public double getB() {
		return b;
	}
}