package org.elsysbg.objectDetectionTools;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;

import org.elsysbg.objectDetectionTools.Blob;
import org.elsysbg.objectDetectionTools.Blob.BlobMassComparator;


public class Segmenter extends Basic {
	private int right, left;
	private double increaseThresh;
	int labels[][];
	int []labelTable;
	
	int label = 1;
	Blob largestBlob;
	
	public Segmenter(String basedir, int index, double increaseThresh){
		super(basedir, index);
		this.increaseThresh = increaseThresh;
	}
	
	private void getBoundaries(){
		//calculate left
		boolean found = false;
		for (int i = 0;i<bufIn.getWidth() && !found; i++){
			for (int j =0; j<bufIn.getHeight() && !found; j++){
				if((bufIn.getRGB(i, j) & 0xffffff) == 0)
					found = true;
				if (found)
					left = i;
			}
		}
		//calculate right
		found = false;
		for(int i =bufIn.getWidth() - 1; i>=0 && !found; i--){
			for(int j = 0; j< bufIn.getHeight() && !found; j++){
				if((bufIn.getRGB(i, j) & 0xffffff) == 0)
					found = true;
				if(found)
					right = i;
			}
		}
	}
	

	/**
	 * find all blobs
	 * @param color
	 * @param minBlobMass
	 * @param maxBlobMass
	 * @param xMin
	 * @param xMax
	 * @param yMin
	 * @param yMax
	 * @return
	 */
	private ArrayList<Blob> findBlobsAlgorithm(int color, int minBlobMass,
			int maxBlobMass, int xMin, int xMax, int yMin, int yMax){
		label = 1;
		int width = bufIn.getWidth();
		int height = bufIn.getHeight();
		
		labels = new int[height][width];
		
		int tableSize = width*height /4;
		labelTable = new int[tableSize];
		int[] xMinTable = new int[tableSize];
		int[] xMaxTable = new int[tableSize];
		int[] yMinTable = new int[tableSize];
		int[] yMaxTable = new int[tableSize];
		int[] massTable = new int[tableSize];
		
		//from top to bottom
		for(int i = yMin; i < yMax; i++){
			//and from left to right
			for(int j= xMin; j< xMax; j++){
				//and set the current pixel with 0
				labels[i][j] = 0;
				
				if((bufIn.getRGB(j, i) & 0xffffff) == color){
					int aLabel = (i>0 && j>0) ? labelTable[labels[i-1][j-1]] : 0;
					int bLabel =  (i > 0) ? labelTable[labels[i - 1][j]] : 0;
					int cLabel = (i > 0 && j < bufIn.getWidth() - 1) ? labelTable[labels[i - 1][j + 1]] : 0;
					int dLabel = (j > 0) ? labelTable[labels[i][j - 1]] : 0;
					
					//search labels with min value
					int min = Integer.MAX_VALUE;
					if (aLabel != 0 && aLabel < min)
						min = aLabel;
					if (bLabel != 0 && bLabel < min)
						min = bLabel;
					if (cLabel != 0 && cLabel < min)
						min = cLabel;
					if (dLabel != 0 && dLabel < min)
						min = dLabel;
					
					//If no neighbours
					if (min == Integer.MAX_VALUE) {
						labels[i][j] = label;
						labelTable[label] = label;

						// Initialise min/max x,y for label
						yMinTable[label] = i;
						yMaxTable[label] = i;
						xMinTable[label] = j;
						xMaxTable[label] = j;
						massTable[label] = 1;

						label++;
					}else{
						// Label pixel with lowest label from neighbours
						labels[i][j] = min;

						// Update min/max x,y for label
						yMaxTable[min] = i;
						massTable[min]++;
						if (j < xMinTable[min])
							xMinTable[min] = j;
						if (j > xMaxTable[min])
							xMaxTable[min] = j;
						
						if (aLabel != 0)
							labelTable[aLabel] = min;
						if (bLabel != 0)
							labelTable[bLabel] = min;
						if (cLabel != 0)
							labelTable[cLabel] = min;
						if (dLabel != 0)
							labelTable[dLabel] = min;
					}
				}
			}
		}
		
		ArrayList<Blob> blobList = new ArrayList<Blob>();
		for(int i = label-1; i>0; i--){
			if (labelTable[i] != i) {
				if (xMaxTable[i] > xMaxTable[labelTable[i]])
					xMaxTable[labelTable[i]] = xMaxTable[i];
				if (xMinTable[i] < xMinTable[labelTable[i]])
					xMinTable[labelTable[i]] = xMinTable[i];
				if (yMaxTable[i] > yMaxTable[labelTable[i]])
					yMaxTable[labelTable[i]] = yMaxTable[i];
				if (yMinTable[i] < yMinTable[labelTable[i]])
					yMinTable[labelTable[i]] = yMinTable[i];
				massTable[labelTable[i]] += massTable[i];


				int l = i;
				while (l != labelTable[l])
					l = labelTable[l];
				labelTable[i] = l;
			}else{
				// Ignore blobs that butt against corners
				if (i == labels[yMin][xMin])
					continue; // Top Left
				if (i == labels[yMin][xMax - 1])
					continue; // Top Right
				if (i == labels[yMax - 1][xMin])
					continue; // Bottom Left
				if (i == labels[yMax - 1][xMax - 1])
					continue; // Bottom Right

				if (massTable[i] >= minBlobMass
						&& (massTable[i] <= maxBlobMass || maxBlobMass == -1)) {
					Blob blob = new Blob(xMinTable[i], xMaxTable[i],
							yMinTable[i], yMaxTable[i], massTable[i],
							labelTable[i]);
					blobList.add(blob);
					System.out.println(blob);
				}
			}
		}
		System.out.println();
		return blobList;
	}
	

	/**
	 * find the largest black blob and save it 
	 * in new image named cleanUp
	 * @throws IOException
	 */
	private void largestBlob() throws IOException{
		openInOutFile("outbox", "_filter", "bmp");
		ArrayList<Blob> blobs = findBlobsAlgorithm(0x000000, 0, -1,
				2 * bufIn.getWidth() / 10, 9 * bufIn.getWidth() / 10,
				0 * bufIn.getHeight() / 10, 10 * bufIn.getHeight() / 10);
		
		int maxMass = 0;
		largestBlob = null;

		for (Blob blob : blobs) {
			if (blob.mass > maxMass) {
				maxMass = blob.mass;
				largestBlob = blob;
			}
		}
		for (int i = 0; i < bufOut.getHeight(); i++) {
			for (int j = 0; j < bufOut.getWidth(); j++) {
				if (largestBlob != null) {

					if (labelTable[labels[i][j]] == labelTable[largestBlob.label]) {
						bufOut.setRGB(j, i, 0x000000);
					} else {
						bufOut.setRGB(j, i, 0xffffff);
					}
				}
			}
		}
		saveOutFile("outbox", "_cleanup", "bmp");
	}
	
	/**
	 * delete all parasite pixels
	 * @throws IOException
	 */
	private void increase() throws IOException{
		openInOutFile("outbox", "_cleanup", "bmp");
		getBoundaries();
		//left
		for(int i = 0; i<left; i++){
			for(int j = 0; j<bufIn.getHeight(); j++){
				bufOut.setRGB(i, j, 0xffffff);
				
			}
		}
		//right
		for (int i = right+1; i<bufIn.getWidth(); i++){
			for (int j= 0; j <bufIn.getHeight(); j++){
				bufOut.setRGB(i, j, 0xffffff);
			}
		}
		
		for(int i = left; i<=right; i++){
			//find local top
			boolean found = false;
			int top = bufIn.getHeight();
			for(int j =0; j<=bufIn.getHeight() && !found; j++){
				if((bufIn.getRGB(i, j)& 0xffffff) == 0){
					found = true;
					top = j;
				}
			}
			
			//find local bottom
			found = false;
			int bottom = 0 ;
			for(int j = bufIn.getHeight() - 1; j>=0 && !found; j--){
				if ((bufIn.getRGB(i, j) & 0xffffff) == 0){
					found = true;
					bottom = j;
				}
			}
			//set top and bottom
			for (int j = 0;j <top; j++){
				bufOut.setRGB(i, j, 0xffffff);
			}
			for(int j= bottom+1; j<bufIn.getHeight(); j++){
				bufOut.setRGB(i, j, 0xffffff);
			}
			
			//count white from top to bottom
			int countw = 0;
			for(int j = top; j<bottom; j++){
				if((bufIn.getRGB(i, j) & 0x00ffffff) != 0)
					countw ++;
			}
			double ratio = countw / (bottom-top +1.0);
			boolean makeblack = ratio < increaseThresh;
			for(int j= top; j<= bottom; j++){
				if (makeblack)
					bufOut.setRGB(i, j, 0);
				else bufOut.setRGB(i, j, bufIn.getRGB(i, j));
			}
		}
		saveOutFile("outbox","_increase", "bmp");
	}
	
	/**
	 * outputs all of the current blobs with a unique color
	 */
	private void dumpBlobs(){
		int rgb[] = new int[label];
		rgb[0] = 0;
		for(int c = 1; c< rgb.length; ++c){
			rgb[c] = 0+ (int)(Math.random()*255 *255 *255);
		}
		for (int y=0; y< bufOut.getHeight(); ++y){
			for( int x= 0; x < bufOut.getWidth(); ++x){
				bufOut.setRGB(x, y, rgb[labelTable[labels[y][x]]]);
			}
		}
	}
	
	/**
	 * this method creates segments that contains the 8 largest white
	 * blobs in increased image
	 * @throws IOException 
	 */
	private void createSegments() throws IOException{
		openInOutFile("outbox", "_increase", "bmp");
		//Here we determinate the white blobs that correspondent to the letters
		// on the license plate
		
		//This finds all the white blobs within the largest black blob
		ArrayList<Blob> blobs = findBlobsAlgorithm(0xffffff, 0, -1,
				largestBlob.xMin + 5, largestBlob.xMax - 5,
				largestBlob.yMin + 5, largestBlob.yMax -5);
		
		//sort the blobs according to their mass
		Collections.sort(blobs, new BlobMassComparator());
		
		closeInFile();
		
		//select the 8 biggest blobs
		blobs = new ArrayList<Blob>(blobs.subList(0, Math.min(8, blobs.size()))); // if we had less than 8 blobs
		
		//sort the blobs according their order of appearance
		Collections.sort(blobs, new Blob.BlobOrderComparator());
		
		//create 8 masks
		for (int i=0; i< blobs.size(); i++){
			openInOutFile("outbox", "_filter", "bmp");
			
			//creates the masked image for blob number i;
			for(int y = 0; y < bufIn.getHeight(); y++){
				for(int x = 0; x < bufIn.getWidth(); x++){
					//check if the current pixel is part of the current blob
					//and if it's true than set its value
					// else set it to black
					if(blobs.get(i).yMin<=y
							&& y <= blobs.get(i).yMax
							&& blobs.get(i).xMin <= x
							&& x <= blobs.get(i).xMax
							&& labelTable[labels[y][x]]== labelTable[blobs.get(i).label]){
						int rgb = bufIn.getRGB(x, y);
						rgb = bufIn.getRGB(x, y);
						bufOut.setRGB(x, y, rgb);
					}else{
						bufOut.setRGB(x, y, 0x000000);
					}
				}
			}
			saveOutFile("outbox", "_" +(i+1)+"seg", "bmp");
		}
		dumpBlobs();
		saveOutFile("outbox", "_"+"dump"+"_seg", "bmp");
		
	}

	public void segment() throws IOException{
		largestBlob();
		increase();
		createSegments();
		
	}

}
