package org.elsysbg.objectDetectionTools;

import java.awt.Color;

public class XYZColor {
	private double x;
	private double y;
	private double z;
	
	
	/**
	 * Construct new XYZ color from given RGB values
	 * @param red
	 * @param green
	 * @param blue
	 */
	public XYZColor(int red, int green, int blue) {
		double xyzRed = rgbToXYZ(red);
		double xyzGreen = rgbToXYZ(green);
		double xyzBlue = rgbToXYZ(blue);
		
		this.x = xyzRed * 0.4124 + xyzGreen * 0.3576 + xyzBlue * 0.1805;
		this.y = xyzRed * 0.2126 + xyzGreen * 0.7152 + xyzBlue * 0.0722;
		this.z = xyzRed * 0.0193 + xyzGreen * 0.1192 + xyzBlue * 0.9505;
		
	}

	/**
	 * construct new XYZ color with given double(XYZ) values
	 * @param x
	 * @param y
	 * @param z
	 */
	public XYZColor(double x,double y,double z){
		this.x = x;
		this.y = y;
		this.z = z;
	}
	
	
	/**
	 * construct new XYZ color from given color 
	 * @param color
	 */
	public XYZColor(Color color){
		this(color.getRed(), color.getGreen(), color.getBlue());
	}
	
	
	
	double rgbToXYZ(double value) {
		value /= 255; // x/(x+y+z) -> x+y+z = 255
		
		if (value >0.04045){
			value = (value+0.055)/1.055;
			value = Math.pow(value, 2.4);
		}else{
			value = value/12.92;
		}
		value*=100;
		return value;
	}

	/**
	 * @return the x
	 */
	public double getX() {
		return x;
	}

	/**
	 * @param x the x to set
	 */
	public void setX(double x) {
		this.x = x;
	}

	/**
	 * @return the y
	 */
	public double getY() {
		return y;
	}

	/**
	 * @param y the y to set
	 */
	public void setY(double y) {
		this.y = y;
	}

	/**
	 * @return the z
	 */
	public double getZ() {
		return z;
	}

	/**
	 * @param z the z to set
	 */
	public void setZ(double z) {
		this.z = z;
	}
}
